(($, Drupal, once) => {

  Drupal.behaviors.GradientBuilder = {

    attach(context, settings) {

      once('GradientBuilder', '.gradient-builder', context).forEach(gradientBuilder => {
        const builderId = gradientBuilder.dataset.builderId;

        // Create a Proxy for this gradient builder.
        const handler = this.createTempStoreHandler(builderId);
        const colorStopsProxy = new Proxy({}, handler);
        this.temporaryStoreMap.set(builderId, colorStopsProxy);
        this.initializeTempstore(builderId);

        this.showPreview(gradientBuilder);
        this.setupDial(builderId);
      });

      // Add color stop button.
      once('AddColorStop', '.add-color-stop', context).forEach(button => {
        button.addEventListener('click', (e) => {
          e.preventDefault();
          this.addColorStop(e.target.closest('.gradient-builder'));
        });
      });

      // Preview button.
      once('Preview', '.preview-gradient', context).forEach(button => {
        button.addEventListener('click', (e) => {
          e.preventDefault();
          this.showPreview(e.target.closest('.gradient-builder'));
        });
      });

      // Text input changes.
      once('TextChanges', '.detail-wrapper input', context).forEach(input => {
        input.addEventListener('input', (e) => {
          e.preventDefault();
          this.textInputChanged(e);
        });
        input.addEventListener('keydown', event => {
          if (event.key === 'Enter') {
            event.preventDefault();
          }
        });
      });

      // Remove stop button
      once('RemoveStop', '.delete-stop', context).forEach(button => {
        button.addEventListener('click', (e) => {
          e.preventDefault();
          const builderWrapper = e.target.closest('.gradient-builder');
          const builderId = builderWrapper.dataset.builderId;
          const stop = this.getStop(builderId);
          this.removeStop(builderId, stop);
          const stopElement = builderWrapper.querySelector('[data-id="' + stop.id + '"]');
          stopElement.remove();
        });
      });
    },

    setupDial(builderId) {
      const gradientBuilder = this.getBuilderWrapper(builderId);
      const dial = gradientBuilder.querySelector('.dial');
      let isDragging = false;
      let currentAngle = this.getAngle(builderId);

      const updateIndicator = () => {
        const indicator = dial.querySelector('.indicator');
        indicator.style.transform = `rotate(${currentAngle}deg)`;
        const angleDisplay = dial.querySelector('.angle-display');
        angleDisplay.textContent = `${Math.round(currentAngle)}°`;
        this.setAngle(builderId, currentAngle);
      };

      dial.addEventListener('pointerdown', (e) => {
        isDragging = true;
        document.body.style.userSelect = 'none';
        const rect = dial.getBoundingClientRect();
        const centerX = rect.left + rect.width / 2;
        const centerY = rect.top + rect.height / 2;
        const deltaX = e.clientX - centerX;
        const deltaY = e.clientY - centerY;
        const angleRad = Math.atan2(deltaY, deltaX);
        currentAngle = (angleRad * 180 / Math.PI + 360) % 360;
        updateIndicator();
      });

      document.addEventListener('pointermove', (e) => {
        if (isDragging) {
          const rect = dial.getBoundingClientRect();
          const centerX = rect.left + rect.width / 2;
          const centerY = rect.top + rect.height / 2;
          const deltaX = e.clientX - centerX;
          const deltaY = e.clientY - centerY;
          const angleRad = Math.atan2(deltaY, deltaX);
          currentAngle = (angleRad * 180 / Math.PI + 360) % 360;
          updateIndicator();
          e.preventDefault();
        }
      });

      document.addEventListener('pointerup', () => {
        isDragging = false;
        document.body.style.userSelect = '';
      });

      updateIndicator();
    },

    textInputChanged(e) {
      const input = e.target;
      const builderWrapper = this.getBuilderWrapperFromElement(input);
      const builderId = builderWrapper.dataset.builderId;
      let stop = this.getStop(builderId);
      switch (e.target.name) {
        case 'hex':
          stop.color = input.value;
          break;
        case'red':
        case'green':
        case'blue':
          const r = builderWrapper.querySelector('[name="red"]').value;
          const g = builderWrapper.querySelector('[name="green"]').value;
          const b = builderWrapper.querySelector('[name="blue"]').value;
          stop.color = this.rgbToHex(r, g, b);
          break;
        case'position':
          stop.position = input.value;
          break;
      }
      this.setStop(builderId, stop);

    },

    hidePreview(wrapper) {
      const colorPicker = wrapper.querySelector('.color-picker-container');
      colorPicker.style.display = null;

      const preview = wrapper.querySelector('.gradient-preview');
      preview.style.display = 'none';

      const previewButton = wrapper.querySelector('.preview-gradient');
      previewButton.style.display = 'inline-block';
    },
    showPreview(wrapper) {
      const colorPicker = wrapper.querySelector('.color-picker-container');
      colorPicker.style.display = 'none';

      const previewButton = wrapper.querySelector('.preview-gradient');
      previewButton.style.display = 'none';
      const builderId = wrapper.dataset.builderId;

      const preview = this.getBuilderWrapper(builderId).querySelector('.gradient-preview');
      preview.style.display = 'block';
    },

    initializeTempstore(builderId) {
      let tempStore = this.temporaryStoreMap.get(builderId);
      tempStore['stops'] = [];
      tempStore['angle'] = 90;

      const gradientDataInput = this.getBuilderInput(builderId);
      const savedValuesString = gradientDataInput.value;
      const savedValues = savedValuesString ? JSON.parse(savedValuesString) : [];
      const builderWrapper = this.getBuilderWrapper(builderId);

      if (Object.keys(savedValues).length > 0 && savedValues['stops'] !== undefined) {
        const stops = savedValues['stops'];
        if (stops.length > 0) {
          this.setAngle(builderId, savedValues['angle']);
          for (const stop of stops) {
            this.addColorStop(builderWrapper, stop.color, stop.position);
          }
          return;
        }
      }
      this.addColorStop(builderWrapper);
    },

    createTempStoreHandler(builderId) {
      return {
        set(target, property, value) {
          target[property] = value;
          // console.log(`Builder ${builderId}: id: ${value.id} color: ${value.color} position: ${value.position}%`);
          Drupal.behaviors.GradientBuilder.update(builderId);
          return true;
        },
        deleteProperty(target, property) {
          if (property in target) {
            delete target[property];
            Drupal.behaviors.GradientBuilder.update(builderId);
            return true;
          }
          return true;
        }
      };
    },

    /**
     * Add color stop.
     *
     * Adds a color stop to the gradient builder slider.
     *
     * @param gradientBuilderWrapper
     *   The gradient builder wrapper.
     * @param color
     *   The color to add.
     * @param position
     *   The position on the slider 0 - 100.
     */
    addColorStop(gradientBuilderWrapper, color = '#0075db', position = 50) {
      const colorStopElements = gradientBuilderWrapper.querySelectorAll('[data-id]');

      // Calculate the next color stop ID.
      let id = null;
      if (colorStopElements.length > 0) {
        id = Math.max(...Array.from(colorStopElements)
          .map(el => parseInt(el.dataset.id, 10))
          .filter(id => !isNaN(id)))
           + 1;
      } else {
        id = 1;
      }
      const stop = {id: id, position: position, color: color};

      const builderId = gradientBuilderWrapper.dataset.builderId;
      this.setStop(builderId, stop);
      this.createStopElement(builderId, id, stop)
      this.activeColorStop[builderId] = id;
      this.openColorPicker(gradientBuilderWrapper);
      this.hidePreview(gradientBuilderWrapper);
    },

    createStopElement(builderId, id, stop) {
      const slider = this.getSlider(builderId);
      const stopElement = document.createElement('div');
      stopElement.className = 'color-stop';
      stopElement.dataset.id = id;
      stopElement.style.left = stop.position + '%';
      stopElement.style.backgroundColor = stop.color;

      // Create and append the label showing the percentage.
      const label = document.createElement('div');
      label.className = 'stop-label';
      label.innerText = stop.position + '%';
      stopElement.appendChild(label);

      // Variables to handle dragging and click detection.
      let isDragging = false;
      let startX, startY;
      let hasMoved = false;
      const clickThreshold = 5; // pixels

      // On mousedown, initialize dragging variables.
      stopElement.addEventListener('mousedown', (e) => {
        // Only respond to left-click.
        if (e.button !== 0) {
          return;
        }
        const colorStopElement = e.currentTarget;
        const wrapper = this.getBuilderWrapperFromElement(e.target);
        if (!wrapper) {
          return;
        }
        const builderId = wrapper.dataset.builderId;
        this.activeColorStop[builderId] = colorStopElement.dataset.id;

        isDragging = true;
        hasMoved = false;
        startX = e.clientX;
        startY = e.clientY;

        e.preventDefault();
      });

      // Update position on mouse move if dragging.
      document.addEventListener('mousemove', (e) => {
        if (!isDragging) {
          return;
        }
        if (!hasMoved && (Math.abs(e.clientX - startX) > clickThreshold || Math.abs(e.clientY - startY) > clickThreshold)) {
          hasMoved = true;
        }
        if (hasMoved) {
          const rect = slider.getBoundingClientRect();
          let position = ((e.clientX - rect.left) / rect.width) * 100;
          position = parseInt(Math.max(0, Math.min(100, position)));
          stop.position = position;
          stopElement.style.left = stop.position + '%';
          label.innerText = stop.position + '%';

          const builderId = this.getBuilderWrapperFromElement(stopElement).dataset.builderId;
          let colorStop = this.getStop(builderId);
          colorStop.position = position;
          this.setStop(builderId, colorStop);
        }
      });

      // On mouseup, if no dragging then trigger the color picker.
      document.addEventListener('mouseup', (e) => {
        if (isDragging) {
          isDragging = false;
          if (!hasMoved) {
            const wrapper = this.getBuilderWrapperFromElement(e.target);
            if (!wrapper) {
              return;
            }
            this.openColorPicker(wrapper);
            this.hidePreview(wrapper);
          }
        }
      });

      slider.appendChild(stopElement);
    },

    openColorPicker: function (builderWrapper) {
      const builderId = builderWrapper.dataset.builderId;
      const container = builderWrapper.querySelector('.color-picker');
      let ignoreColorPickerInitialization = false;
      const colorChangedCallback = (color) => {
        if (ignoreColorPickerInitialization) {
          ignoreColorPickerInitialization = false;
          return;
        }
        let stop = this.getStop(builderId);
        stop.color = color;
        this.setStop(builderId, stop);
      };
      ignoreColorPickerInitialization = true;
      this.colorPicker[builderId] = $.farbtastic(container, colorChangedCallback);
      const stop = this.getStop(builderWrapper.dataset.builderId);
      this.colorPicker[builderId].setColor(stop.color);
    },

    update(builderId) {
      const stops = this.getColorStops(builderId);
      if (!stops.length) {
        return;
      }

      this.updateUserInput(builderId);
      this.updateSliderBackground(builderId);
      this.updateColorStopDetails(builderId);
      this.updateStopElement(builderId);
      this.updatePreview(builderId);
    },

    updateStopElement(builderId) {
      const stop = this.getStop(builderId);
      if (stop && stop.id) {
        const stopElement = this.getBuilderWrapper(builderId).querySelector('.color-stop[data-id="' + stop.id + '"]');
        const newPosition = stop.position + '%';
        stopElement.style.left = newPosition;
        stopElement.style.backgroundColor = stop.color;
        stopElement.querySelector('.stop-label').innerHTML = newPosition;

        this.colorPicker[builderId].setColor(stop.color);
      }
    },

    updateColorStopDetails(builderId) {
      const stop = this.getStop(builderId);
      if (!stop) {
        return;
      }
      const wrapper = this.getBuilderWrapper(builderId);
      const hexTextInput = wrapper.querySelector('[name="hex"]');
      hexTextInput.value = stop.color;
      const rgb = this.hexToRgb(stop.color);
      const redTextInput = wrapper.querySelector('[name="red"]');
      redTextInput.value = rgb.r;
      const greenTextInput = wrapper.querySelector('[name="green"]');
      greenTextInput.value = rgb.g;
      const blueTextInput = wrapper.querySelector('[name="blue"]');
      blueTextInput.value = rgb.b;
      const positionTextInput = wrapper.querySelector('[name="position"]');
      positionTextInput.value = stop.position;
    },

    // Update the slider's background to show the gradient and update the hidden input.
    updateSliderBackground(builderId) {
      const colorStops = this.getColorStops(builderId);
      const slider = this.getSlider(builderId);
      if (colorStops.length === 0) {
        slider.style.background = '#ddd';
        return;
      }
      const gradientString = this.getGradientString(builderId);
      slider.style.background = `linear-gradient(90deg, ${gradientString})`;
    },

    updatePreview(builderId) {
      const preview = this.getBuilderWrapper(builderId).querySelector('.gradient-preview');

      preview.style.background = this.getGradient(builderId);
    },

    updateUserInput(builderId) {
      const sortedStops = this.getSortedStops(builderId);
      const gradientDataInput = this.getBuilderInput(builderId);
      const angle = this.getAngle(builderId);
      gradientDataInput.value = JSON.stringify({stops: sortedStops, angle: parseInt(angle)});
    },

    getSortedStops(builderId) {
      const colorStops = this.getColorStops(builderId);
      // Sort colorStops by position.
      return Object.values(colorStops).sort((a, b) => a.position - b.position);
    },

    getGradientString(builderId) {
      // Sort colorStops by position.
      const sortedStops = this.getSortedStops(builderId);
      return sortedStops
        .map(stop => `${stop.color} ${stop.position}%`)
        .join(', ');
    },

    getGradient(builderId) {
      const angle = this.getAngle(builderId);
      const gradientString = this.getGradientString(builderId);
      return `linear-gradient(${angle}deg, ${gradientString})`;
    },

    getBuilderInput(builderId) {
      return this.getBuilderWrapper(builderId).querySelector('[name="gradient"]');
    },

    getBuilderWrapperFromElement(element) {
      return element.closest('.gradient-builder');
    },

    getBuilderWrapper(builderId) {
      return document.querySelector('[data-builder-id="' + builderId + '"]');
    },

    getSlider(builderId) {
      return this.getBuilderWrapper(builderId).querySelector('.gradient-slider');
    },

    getAngle(builderId) {
      const proxy = this.temporaryStoreMap.get(builderId);
      return proxy['angle'];
    },

    getColorStops(builderId) {
      const proxy = this.temporaryStoreMap.get(builderId);
      return proxy['stops'];
    },

    getStop(builderId, id) {
      if (!id) {
        id = Drupal.behaviors.GradientBuilder.activeColorStop[builderId];
      }
      return this.getColorStops(builderId)[id]
    },

    setStop(builderId, stop) {
      let tempStore = this.temporaryStoreMap.get(builderId);
      let stops = tempStore['stops'];
      stops[stop.id] = stop;
      tempStore['stops'] = stops;
    },

    setAngle(builderId, angle) {
      let tempStore = this.temporaryStoreMap.get(builderId);
      tempStore['angle'] = angle;
    },

    removeStop(builderId, stop) {
      let tempStore = this.temporaryStoreMap.get(builderId);
      let stops = tempStore['stops'];
      delete stops[stop.id];
      tempStore['stops'] = stops;
    },

    hexToRgb(hex) {
      // Remove # if present
      hex = hex.replace(/^#/, '');

      // Handle both 3-digit and 6-digit formats
      if (hex.length === 3) {
        hex = hex[0] + hex[0] + hex[1] + hex[1] + hex[2] + hex[2];
      }

      return {
        r: parseInt(hex.substring(0, 2), 16),
        g: parseInt(hex.substring(2, 4), 16),
        b: parseInt(hex.substring(4, 6), 16)
      };
    },

    rgbToHex(r, g, b) {
      // Ensure values are integers between 0-255
      r = Math.max(0, Math.min(255, Math.round(r)));
      g = Math.max(0, Math.min(255, Math.round(g)));
      b = Math.max(0, Math.min(255, Math.round(b)));

      // Convert to hex and pad with zeros if needed
      return '#' +
        r.toString(16).padStart(2, '0') +
        g.toString(16).padStart(2, '0') +
        b.toString(16).padStart(2, '0');
    },
  };


  Drupal.behaviors.GradientBuilder.temporaryStoreMap = Drupal.behaviors.GradientBuilder.temporaryStoreMap || new Map();
  Drupal.behaviors.GradientBuilder.activeColorStop = Drupal.behaviors.GradientBuilder.activeColorStop || {};
  Drupal.behaviors.GradientBuilder.colorPicker = Drupal.behaviors.GradientBuilder.colorPicker || {};

})(jQuery, Drupal, once);


